@@ -255,7 +255,10 @@ class AlphaFlask(Flask):
         routes = self.conf.get("routes")
         if routes is not None:
             for route in routes:
-                module = importlib.import_module(route)
+                try:
+                    module = importlib.import_module(route)
+                except Exception as ex:
+                    self.log.critical(f"Cannot import routes from {route}",ex=ex)
         self.cache_dir = self.conf.get("directories/cache")
 
         if routes is None:
@@ -546,10 +549,10 @@ class AlphaFlask(Flask):
 
         default_route = Route(request_uuid, request.full_path, [], request)
         if request_uuid not in self.routes_objects:
-            self.log.critical(f"Cannot get route for {request_uuid}", monitor="DB")
+            self.log.critical(f"Cannot get route for {request_uuid}")
             return default_route
 
-        self.log.critical(f"Issue with route {request_uuid}", monitor="DB")
+        self.log.critical(f"Issue with route {request_uuid}")
         return default_route
 
     def get_path(self):
