@@ -204,7 +204,7 @@ class AlphaConfig(AlphaClass):
                         tmp_config, configurations[configuration_name], replace=True
                     )
                     self.info(
-                        f"{'Start with' if index == 0 else 'Merge'} configuration {configuration_name}"
+                        f"{'Using base' if index == 0 else 'Merging'} configuration {configuration_name}"
                     )
                     index += 1
                 except ValueError as ex:
