@@ -1,6 +1,5 @@
 # import mysql.connector
-import inspect, os, re, itertools
-import typing
+import enum, re, itertools, typing
 
 import numpy as np
 
@@ -28,6 +27,7 @@ from sqlalchemy.engine.reflection import Inspector
 from sqlalchemy.exc import OperationalError
 from sqlalchemy.orm.base import object_mapper
 from sqlalchemy.orm.exc import UnmappedInstanceError
+from collections.abc import Iterable
 
 from time import sleep
 import logging
@@ -38,8 +38,9 @@ from ...models.logger import AlphaLogger
 from ...libs import dict_lib, py_lib
 from ...models.main import AlphaException
 from .operators import Operators
+from sqlalchemy.sql.elements import Null
 
-def default_return(results, default=None, columns:dict=None):
+def default_return(results, default=None, columns:dict=None, page:int=None, full_count:int=None):
     results = results if results is not None else default
     columns = {} if columns is None else {x if not hasattr(x,"key") else x.key():y for x,y in columns.items()}
     
@@ -48,6 +49,8 @@ def default_return(results, default=None, columns:dict=None):
             results = {x if x not in columns else columns[x]:y for x,y in results.items()}
         elif type(results) == list and len(results) != 0 and type(results)[0] == dict:
             results = [{x if x not in columns else columns[x]:y for x,y in r.items()} for r in results]
+    if page is not None:
+        return (results, full_count)
     return results
 
 def get_conditions_from_dict(values: dict, model=None, optional: bool = False):
@@ -60,6 +63,8 @@ def get_conditions_from_dict(values: dict, model=None, optional: bool = False):
             value = list(value)
         elif type(value) == dict:
             for k, v in value.items():
+                if issubclass(type(v), enum.Enum):
+                    v = v.value
                 if optional and v is None:
                     continue
 
@@ -68,13 +73,39 @@ def get_conditions_from_dict(values: dict, model=None, optional: bool = False):
                 elif Operators.DIFFERENT.equals(k) or Operators.NOT.equals(k):
                     conditions.append(key != v)
                 elif Operators.LIKE.equals(k):
-                    conditions.append(key.like(v))
+                    if not isinstance(v, Null):
+                        conditions.append(key.like(v))
+                    else:
+                        conditions.append(key==v)
                 elif Operators.NOT_LIKE.equals(k):
-                    conditions.append(~key.like(v))
+                    if not isinstance(v, Null):
+                        conditions.append(~key.like(v))
+                    else:
+                        conditions.append(key!=v)
                 elif Operators.ILIKE.equals(k):
-                    conditions.append(key.ilike(v))
+                    if not isinstance(v, Null):
+                        conditions.append(key.ilike(v))
+                    else:
+                        conditions.append(key==v)
                 elif Operators.NOT_ILIKE.equals(k):
-                    conditions.append(~key.ilike(v))
+                    if not isinstance(v, Null):
+                        conditions.append(~key.ilike(v))
+                    else:
+                        conditions.append(key!=v)
+                elif Operators.BETWEEN.equals(k):
+                    if len(v) != 2:
+                        continue
+                    if v[0] is not None:
+                        conditions.append(key > v[0])
+                    if v[1] is not None:
+                        conditions.append(key < v[1])
+                elif Operators.BETWEEN_OR_EQUAL.equals(k):
+                    if len(v) != 2:
+                        continue
+                    if v[0] is not None:
+                        conditions.append(key >= v[0])
+                    if v[1] is not None:
+                        conditions.append(key <= v[1])
                 elif Operators.SUPERIOR.equals(k):
                     conditions.append(key > v)
                 elif Operators.INFERIOR.equals(k):
@@ -84,9 +115,9 @@ def get_conditions_from_dict(values: dict, model=None, optional: bool = False):
                 elif Operators.INFERIOR_OR_EQUAL.equals(k):
                     conditions.append(key <= v)
                 elif Operators.NOT_IN.equals(k):
-                    conditions.append(key.notin_(v))
+                    conditions.append(key.notin_(v if isinstance(v, Iterable) else [v]))
                 elif Operators.IN.equals(k):
-                    conditions.append(key.in_(v))
+                    conditions.append(key.in_(v if isinstance(v, Iterable) else [v]))
                 elif Operators.HAS.equals(k):
                     v = get_filters(v, None, optional=optional)
                     for condition in v:
@@ -326,6 +357,8 @@ class AlphaDatabaseCore(SQLAlchemy):
         query = model.query
 
         if columns is not None:
+            if type(columns) != list and type(columns) != typing.List:
+                columns = [columns]
             ccs = []
             for column in columns:
                 """if type(column) != str:
@@ -389,10 +422,6 @@ class AlphaDatabase(AlphaDatabaseCore):
             return self.get_query_results(
                 query, values, unique=False, bind=bind, close=close
             )
-
-        """session = self.get_engine().connect()
-        trans = session.begin()"""
-
         try:
             if multi:
                 for value in values:
@@ -412,7 +441,7 @@ class AlphaDatabase(AlphaDatabaseCore):
             return True
         except Exception as ex:
             self.log.error(ex)
-            return False
+            raise ex
 
     def get(self, query, values=None, unique=False, bind: str = None, log=None):
         return self.get_query_results(
@@ -429,60 +458,60 @@ class AlphaDatabase(AlphaDatabaseCore):
 
         if log is None:
             log = self.log
-
-        rows = []
+        
+        if values is not None:
+            if type(values) == list and len(values) != 0:
+                dict_values = {}
+                for i, val in enumerate(values):
+                    if type(val) == dict:
+                        query = query.replace(
+                            ":%s" % list(val.keys())[0], f":p{i}", 1
+                        )
+                        dict_values[f"p{i}"] = list(val.values())[0]
+                    else:
+                        query = query.replace("?", f":p{i}", 1)
+                        dict_values[f"p{i}"] = val
+                values = dict_values
+        
         try:
-            if values is not None:
-                if type(values) == list and len(values) != 0:
-                    dict_values = {}
-                    for i, val in enumerate(values):
-                        if type(val) == dict:
-                            query = query.replace(
-                                ":%s" % list(val.keys())[0], f":p{i}", 1
-                            )
-                            dict_values[f"p{i}"] = list(val.values())[0]
-                        else:
-                            query = query.replace("?", f":p{i}", 1)
-                            dict_values[f"p{i}"] = val
-                    values = dict_values
-
-                resultproxy = session.execute(query, values)
-            else:
-                resultproxy = session.execute(query)
-            results = []
-            for rowproxy in resultproxy:
-                if hasattr(rowproxy, "items"):
-                    columns = {column: value for column, value in rowproxy.items()}
-                else:
-                    columns = [x for x in rowproxy]
-                results.append(columns)
-
-            if not unique:
-                rows = [Row(x) for x in results]
-            else:
-                rows = [
-                    value[0] if not hasattr(value, "keys") else list(value.values())[0]
-                    for value in results
-                ]
-            self.query_str = get_compiled_query(query).replace("\n", "")
+            resultproxy = session.execute(query, values) if values is not None else session.execute(query)
         except Exception as ex:
+            if log is not None:
+                log.error(ex)
+            raise ex
+        
+        results = []
+        for rowproxy in resultproxy:
+            if hasattr(rowproxy, "items"):
+                columns = {column: value for column, value in rowproxy.items()}
+            else:
+                columns = [x for x in rowproxy]
+            results.append(columns)
+
+        if not unique:
+            rows = [Row(x) for x in results]
+        else:
+            rows = [
+                value[0] if not hasattr(value, "keys") else list(value.values())[0]
+                for value in results
+            ]
+        self.query_str = get_compiled_query(query).replace("\n", "")
+        
+        """
+        except Exception as err:
+            stack = inspect.stack()
+            parentframe = stack[1]
+            module = inspect.getmodule(parentframe[0])
+            root = os.path.abspath(module.__file__).replace(module.__file__, "")
+            error_message = "In file {} line {}:\n {} \n\n {}".format(
+                parentframe.filename,
+                parentframe.lineno,
+                "\n".join(parentframe.code_context),
+                err,
+            )
             if self.log is not None:
-                self.log.error(ex)
-            """
-            except Exception as err:
-                stack = inspect.stack()
-                parentframe = stack[1]
-                module = inspect.getmodule(parentframe[0])
-                root = os.path.abspath(module.__file__).replace(module.__file__, "")
-                error_message = "In file {} line {}:\n {} \n\n {}".format(
-                    parentframe.filename,
-                    parentframe.lineno,
-                    "\n".join(parentframe.code_context),
-                    err,
-                )
-                if self.log is not None:
-                    self.log.error(error_message)
-            """
+                self.log.error(error_message)
+        """
         if close:
             session.close()
 
@@ -834,6 +863,7 @@ class AlphaDatabase(AlphaDatabaseCore):
         unique: InstrumentedAttribute = None,
         count: bool = False,
         limit: int = None,
+        filters: list = None,
         optional_filters: list = None,
         order_by=None,
         order_by_direction=None,
@@ -849,8 +879,12 @@ class AlphaDatabase(AlphaDatabaseCore):
         default=None,
         columns: dict = None
     ):
+        full_count = None
         if dataclass is not None:
             json = True
+        if filters is not None:
+            filters = get_filters(filters, model=None, optional=False)
+            query = query.filter(and_(*filters))
         if optional_filters is not None:
             optional_filters = get_filters(optional_filters, model=None, optional=True)
             query = query.filter(and_(*optional_filters))
@@ -872,7 +906,7 @@ class AlphaDatabase(AlphaDatabaseCore):
             results = query.count()
             self.query_str = get_compiled_query(query).replace("\n", "")
             self.log.debug(self.query_str)
-            return default_return(results, default=default, columns=columns)
+            return default_return(results, default=default, columns=columns, page=page,full_count=full_count)
 
         try:
             results = query.all() if not first else query.first()
@@ -891,7 +925,7 @@ class AlphaDatabase(AlphaDatabaseCore):
         if not json:
             self.query_str = get_compiled_query(query).replace("\n", "")
             self.log.debug(self.query_str, level=2)
-            return default_return(results if page is None else (results, full_count), default=default, columns=columns)
+            return default_return(results, default=default, columns=columns, page=page,full_count=full_count)
         
         results_json = {}
         if schema is None and model is not None:
@@ -927,26 +961,22 @@ class AlphaDatabase(AlphaDatabaseCore):
                         []
                         if len(results_json) == 0
                         else [x[unique] for x in results_json]
-                    ), default=default, columns=columns)
+                    ), default=default, columns=columns, page=page,full_count=full_count)
                 else:
                     return default_return((
                         results_json[unique]
-                        if page is None
-                        else (results_json, full_count)
-                    ), default=default, columns=columns)
+                    ), default=default, columns=columns, page=page,full_count=full_count)
             else:
                 if not first:
                     return default_return((
                         []
                         if len(results_json) == 0
-                        else [x[unique.key] for x in results_json]
-                    ), default=default, columns=columns)
+                        else [x[unique.key] for x in results_json if unique.key in x]
+                    ), default=default, columns=columns, page=page,full_count=full_count)
                 else:
                     return default_return((
-                        results_json[unique.key]
-                        if page is None
-                        else (results_json, full_count)
-                    ), default=default, columns=columns)
+                        results_json[unique.key] if unique.key in results_json else None
+                    ), default=default, columns=columns, page=page,full_count=full_count)
         """if disabled_relationships and not json:
             if type(results_json) == dict:
                 results_json = model(**results_json)
@@ -954,12 +984,12 @@ class AlphaDatabase(AlphaDatabaseCore):
                 results_json = [model(**x) for x in results_json]"""
         if dataclass is not None:
             if len(results_json) == 0:
-                return default
+                return default_return([] if not first else default, default=default, columns=columns, page=page,full_count=full_count)
             if not first:
                 results_json = [dataclass.auto_map_from_dict(r) for r in results_json]
             else:
                 results_json = dataclass.auto_map_from_dict(results_json)
-        return default_return(results_json if page is None else (results_json, full_count), default=default, columns=columns)
+        return default_return(results_json, default=default, columns=columns, page=page,full_count=full_count)
 
     def update(
         self,
