@@ -8,7 +8,7 @@ from typing_extensions import Annotated
 from ..logger import AlphaLogger
 from ...libs import py_lib, json_lib, string_lib
 
-
+from ._exception import AlphaException
 class AlphaEnum(enum.Enum):
     def __str__(self):
         return str(self.value)
@@ -450,8 +450,44 @@ class AutoMapping(AlphaClass):
                 )
         return field_values
 
-
+def convert_value_from_field(field_type, value, automap:bool=False):
+    if value is None:
+        return None
+    if is_dataclass(field_type):
+        return field_type.map_from_dict(value, automap=automap)
+    else:
+        try:
+            return field_type(value)
+        except:
+            return value
+        
 class AlphaDataclass(AlphaClass):
+    def compare(self, other, attrs_list: List[str], attrs_eq: Dict[str, str] = None) -> bool:
+        if not isinstance(other, self.__class__):
+            raise AlphaException(f"Cannot compare {self.__class__} object with {other.__class__} object")
+        
+        [attrs_list.append(attr) for attr in (attrs_eq if attrs_eq is not None else [])]
+        
+        compared = []
+        for attr in attrs_list:
+            if hasattr(self, attr):                
+                self_attr, other_attr = getattr(self, attr), getattr(other, attr)
+                
+                custom_eq = attrs_eq[attr] if attrs_eq is not None and attr in attrs_eq else None
+                
+                if self_attr is None and other_attr is not None:
+                    compared.append(False)
+                elif self_attr is None and other_attr is None:
+                    compared.append(True)
+                elif custom_eq is not None:
+                    compared.append(getattr(self_attr,custom_eq)(other_attr))
+                else:
+                    compared.append(getattr(self_attr,"__eq__")(other_attr))
+            else:
+                raise AlphaException(f"{self.__class__} object has no attribute {attr}")
+
+        return all(compared)
+
     def update_from_auto_map_from_dict(
         self,
         dict_values: Dict[str, object],
@@ -520,14 +556,23 @@ class AlphaDataclass(AlphaClass):
     ):
         if automap and hasattr(dataclass_type, "auto_map_from_dict"):
             return dataclass_type.auto_map_from_dict(dict_values=dict_value)
-
+        
+        if not py_lib.is_subtype(type(dict_value), Dict) and not py_lib.is_subtype(type(dict_value), dict) and not type(dict_value) == dict:
+            return None
+        if dict_value is None:
+            return None
+        
         field_values = {}
         for key, value in dict_value.items():
             if key in dataclass_type.__dataclass_fields__:
                 field = dataclass_type.__dataclass_fields__[key]
                 field_type = field.type
-                if is_dataclass(field_type):
-                    field_values[key] = field_type.map_from_dict(value)
+                
+                if py_lib.is_subtype(field_type, List) or py_lib.is_subtype(field_type, list):
+                    if value is not None:
+                        field_values[key] = [convert_value_from_field(py_lib.get_subtype(field_type), v, automap=automap) for v in value]
+                    else:
+                        field_values[key] = []
                 else:
-                    field_values[key] = value
+                    field_values[key] = convert_value_from_field(field_type, value, automap=automap)
         return dataclass_type(**field_values)
